@@ -112,7 +112,6 @@ public:
 	/** \brief Initialize an empty list. */
 	DList() : head(0), tail(0), listLen(0) {}
 
-#ifdef DOUBLELIST_VALUE
 	/** 
 	 * \brief Perform a deep copy of the list.
 	 * 
@@ -121,6 +120,7 @@ public:
 	 */
 	DList(const DList &other);
 
+#ifdef DOUBLELIST_VALUE
 	/**
 	 * \brief Clear the double list contents.
 	 *
@@ -138,12 +138,16 @@ public:
 	 * \returns A reference to this.
 	 */
 	DList &operator=(const DList &other);
-#else
-	/** 
-	 * \brief Perform a shallow copy of the list. 
-	 */
-	DList(const DList &other);
 
+	/**
+	 * \brief Transfer the contents of another list into this list.
+	 *
+	 * The elements of the other list moved in. The other list will be empty
+	 * afterwards.  If this list contains any elements before the copy, then
+	 * they are deleted. 
+	 */
+	void transfer(DList &other);
+#else
 	/**
 	 * \brief Abandon all elements in the list. 
 	 *
@@ -152,36 +156,25 @@ public:
 	~DList() {}
 
 	/**
-	 * \brief Perform a shallow copy of the list.
+	 * \brief Perform a deep copy of the list.
 	 *
-	 * Performs a shallow copy of another list into this list. If this list is
-	 * non-empty then its contents are abandoned (not freed).
+	 * The elements of the other list are duplicated and put into this list.
+	 * Each list item is created using the copy constructor. If this list
+	 * contains any elements before the copy, they are abandoned.
 	 *
 	 * \returns A reference to this.
 	 */
 	DList &operator=(const DList &other);
-#endif
 
 	/**
-	 * \brief Shallow copy another list into this list.
+	 * \brief Transfer the contents of another list into this list.
 	 *
-	 * The elements of the other list are copied in by reference. The two
-	 * lists will share the same list elements. If this list contains any
-	 * elements before the copy, then they are abandoned. If either of the
-	 * lists are modified after a shallow copy, the other list may become
-	 * corrupted.
+	 * The elements of the other list moved in. The other list will be empty
+	 * afterwards.  If this list contains any elements before the copy, they
+	 * are abandoned. 
 	 */
-	void shallowCopy(const DList &other);
-
-
-	/** 
-	 * \brief Perform a deep copy of the list. 
-	 *
-	 * Each element is duplicated for the new list. Copy constructors are used
-	 * to create the new elements. If this list contains items, they are first
-	 * deleted.
-	 */
-	void deepCopy(const DList &other);
+	void transfer(DList &other);
+#endif
 
 
 #ifdef DOUBLELIST_VALUE
@@ -424,8 +417,6 @@ public:
 	IterLast last()    { return IterLast(*this); }
 };
 
-#ifdef DOUBLELIST_VALUE
-
 /* Copy constructor, does a deep copy of other. */
 template <DLMEL_TEMPDEF> DList<DLMEL_TEMPUSE>::
 		DList(const DList<DLMEL_TEMPUSE> &other) :
@@ -438,12 +429,14 @@ template <DLMEL_TEMPDEF> DList<DLMEL_TEMPUSE>::
 	}
 }
 
+#ifdef DOUBLELIST_VALUE
+
 /* Assignement operator does deep copy. */
 template <DLMEL_TEMPDEF> DList<DLMEL_TEMPUSE> &DList<DLMEL_TEMPUSE>::
 		operator=(const DList &other)
 {
-	/* Loose the old list. The value list assumes items were allocated on the
-	 * heap by ourselves. It assumes ownership. */
+	/* Free the old list. The value list assumes items were allocated on the
+	 * heap by itself. */
 	empty();
 
 	Element *el = other.head;
@@ -454,53 +447,45 @@ template <DLMEL_TEMPDEF> DList<DLMEL_TEMPUSE> &DList<DLMEL_TEMPUSE>::
 	return *this;
 }
 
+template <DLMEL_TEMPDEF> void DList<DLMEL_TEMPUSE>::
+		transfer(DList &other)
+{
+	/* Free the old list. The value list assumes items were allocated on the
+	 * heap by itself. */
+	empty();
+
+	head = other.head;
+	tail = other.tail;
+	listLen = other.listLen;
+
+	other.abandon();
+}
+
 #else 
 
-/* Copy constructor, does a shallow copy. */
-template <DLMEL_TEMPDEF> DList<DLMEL_TEMPUSE>::
-		DList(const DList<DLMEL_TEMPUSE> &other)
-:
-	head(other.head), 
-	tail(other.tail), 
-	listLen(other.listLen)
-{
-}
-
-/* Assignment does a shallow copy. */
+/* Assignement operator does deep copy. */
 template <DLMEL_TEMPDEF> DList<DLMEL_TEMPUSE> &DList<DLMEL_TEMPUSE>::
 		operator=(const DList &other)
 {
-	head = other.head;
-	tail = other.tail;
-	listLen = other.listLen;
-	return *this;
-}
-#endif
-
-/* Explicit shallow copy. */
-template <DLMEL_TEMPDEF> void DList<DLMEL_TEMPUSE>::
-		shallowCopy(const DList &other)
-{
-	head = other.head;
-	tail = other.tail;
-	listLen = other.listLen;
-}
-
-/* Deep copy another list in. Empties the current list. */
-template <DLMEL_TEMPDEF> void DList<DLMEL_TEMPUSE>::
-		deepCopy(const DList &other)
-{
-	/* Loose the old list. The value list assumes items were allocated on the
-	 * heap by ourselves. It assumes ownership. */
-	empty();
-
 	Element *el = other.head;
 	while( el != 0 ) {
 		append( new Element(*el) );
 		el = el->BASE_EL(next);
 	}
+	return *this;
 }
 
+template <DLMEL_TEMPDEF> void DList<DLMEL_TEMPUSE>::
+		transfer(DList &other)
+{
+	head = other.head;
+	tail = other.tail;
+	listLen = other.listLen;
+
+	other.abandon();
+}
+
+#endif
 
 #ifdef DOUBLELIST_VALUE
 
