@@ -20,16 +20,14 @@
  */
 
 #include <assert.h>
+#include <iostream>
 
 #include "fsmgraph.h"
 #include "mergesort.h"
 #include "parsedata.h"
 
-#ifdef LOG_CONDS
-#include <iostream>
 using std::cerr;
 using std::endl;
-#endif
 
 /* Make a new state. The new state will be put on the graph's
  * list of state. The new state can be created final or non final. */
@@ -55,7 +53,7 @@ StateAp *FsmAp::addState()
  * machine will be made that has len+1 states with one transition between each
  * state for each integer in str. IsSigned determines if the integers are to
  * be considered as signed or unsigned ints. */
-void FsmAp::concatFsm( long *str, int len )
+void FsmAp::concatFsm( Key *str, int len )
 {
 	/* Make the first state and set it as the start state. */
 	StateAp *last = addState();
@@ -73,7 +71,7 @@ void FsmAp::concatFsm( long *str, int len )
 }
 
 /* Case insensitive version of concatFsm. */
-void FsmAp::concatFsmCI( long *str, int len )
+void FsmAp::concatFsmCI( Key *str, int len )
 {
 	/* Make the first state and set it as the start state. */
 	StateAp *last = addState();
@@ -83,17 +81,15 @@ void FsmAp::concatFsmCI( long *str, int len )
 	for ( int i = 0; i < len; i++ ) {
 		StateAp *newState = addState();
 
-		if ( 'a' <= str[i] && str[i] <= 'z' ) {
-			long item = 'A' + ( str[i] - 'a' );
-			attachNewTrans( last, newState, item, item );
-		}
+		KeySet keySet;
+		if ( str[i].isLower() )
+			keySet.insert( str[i].toUpper() );
+		if ( str[i].isUpper() )
+			keySet.insert( str[i].toLower() );
+		keySet.insert( str[i] );
 
-		attachNewTrans( last, newState, str[i], str[i] );
-
-		if ( 'A' <= str[i] && str[i] <= 'Z' ) {
-			long item = 'a' + ( str[i] - 'A' );
-			attachNewTrans( last, newState, item, item );
-		}
+		for ( int i = 0; i < keySet.length(); i++ )
+			attachNewTrans( last, newState, keySet[i], keySet[i] );
 
 		last = newState;
 	}
@@ -105,7 +101,7 @@ void FsmAp::concatFsmCI( long *str, int len )
 /* Construct a machine that matches one character.  A new machine will be made
  * that has two states with a single transition between the states. IsSigned
  * determines if the integers are to be considered as signed or unsigned ints. */
-void FsmAp::concatFsm( long chr )
+void FsmAp::concatFsm( Key chr )
 {
 	/* Two states first start, second final. */
 	setStartState( addState() );
@@ -121,7 +117,7 @@ void FsmAp::concatFsm( long chr )
  * be made that has two states and len transitions between the them. The set
  * should be ordered correctly accroding to KeyOps and should not contain
  * any duplicates. */
-void FsmAp::orFsm( long *set, int len )
+void FsmAp::orFsm( Key *set, int len )
 {
 	/* Two states first start, second final. */
 	setStartState( addState() );
@@ -130,7 +126,7 @@ void FsmAp::orFsm( long *set, int len )
 	setFinState( end );
 
 	for ( int i = 1; i < len; i++ )
-		assert( keyOps->lt( set[i-1], set[i] ) );
+		assert( set[i-1] < set[i] );
 
 	/* Attach on all the integers in the given string of ints. */
 	for ( int i = 0; i < len; i++ )
@@ -142,7 +138,7 @@ void FsmAp::orFsm( long *set, int len )
  * match any characters from low to high inclusive. Low should be less than or
  * equal to high otherwise undefined behaviour results.  IsSigned determines
  * if the integers are to be considered as signed or unsigned ints. */
-void FsmAp::rangeFsm( long low, long high )
+void FsmAp::rangeFsm( Key low, Key high )
 {
 	/* Two states first start, second final. */
 	setStartState( addState() );
@@ -155,7 +151,7 @@ void FsmAp::rangeFsm( long low, long high )
 }
 
 /* Construct a machine that a repeated range of characters.  */
-void FsmAp::rangeStarFsm( long low, long high)
+void FsmAp::rangeStarFsm( Key low, Key high)
 {
 	/* One state which is final and is the start state. */
 	setStartState( addState() );
@@ -187,6 +183,18 @@ void FsmAp::emptyFsm( )
 	setStartState( addState() );
 }
 
+void FsmAp::transferOutData( StateAp *destState, StateAp *srcState )
+{
+	for ( TransList::Iter trans = destState->outList; trans.lte(); trans++ ) {
+		if ( trans->toState != 0 ) {
+			/* Get the actions data from the outActionTable. */
+			trans->actionTable.setActions( srcState->outActionTable );
+
+			/* Get the priorities from the outPriorTable. */
+			trans->priorTable.setPriors( srcState->outPriorTable );
+		}
+	}
+}
 
 /* Kleene star operator. Makes this machine the kleene star of itself. Any
  * transitions made going out of the machine and back into itself will be
@@ -200,13 +208,6 @@ void FsmAp::starOp( )
 	/* Turn on misfit accounting to possibly catch the old start state. */
 	setMisfitAccounting( true );
 
-	/* Get a copy of the final state set before creating the new start state.
-	 * It may get set final and we don't want the start state to be included in
-	 * the final state set. If it is included in the final state set then all
-	 * the final states after it get the transitions of the start state doubled
-	 * up. That's incorrect.*/
-	StateSet finStateSetCopy( finStateSet );
-
 	/* Create the new new start state. It will be set final after the merging
 	 * of the final states with the start state is complete. */
 	StateAp *prevStartState = startState;
@@ -214,18 +215,23 @@ void FsmAp::starOp( )
 	setStartState( addState() );
 
 	/* Merge the new start state with the old one to isolate it. */
-	mergeStates( md, startState, prevStartState, false );
+	mergeStates( md, startState, prevStartState );
 
-	/* For all the final states, merge with the new start state. */
-	StateAp **st = finStateSetCopy.data;
-	int nst = finStateSetCopy.length();
-	for (int i = 0; i < nst; i++, st++)
-		mergeStates( md, *st, startState, true );
+	/* Merge the start state into all final states. Except the start state on
+	 * the first pass. If the start state is set final we will be doubling up
+	 * its transitions, which will get transfered to any final states that
+	 * follow it in the final state set. This will be determined by the order
+	 * of items in the final state set. To prevent this we just merge with the
+	 * start on a second pass. */
+	for ( StateSet::Iter st = finStateSet; st.lte(); st++ ) {
+		if ( *st != startState )
+			mergeStatesLeaving( md, *st, startState );
+	}
 
 	/* Now it is safe to merge the start state with itself (provided it
 	 * is set final). */
-	if ( startState->stateBits & SB_ISFINAL )
-		mergeStates( md, startState, startState, true );
+	if ( startState->isFinState() )
+		mergeStatesLeaving( md, startState, startState );
 
 	/* Now ensure the new start state is a final state. */
 	setFinState( startState );
@@ -308,6 +314,7 @@ void FsmAp::optionalRepeatOp( int times )
 	doConcat( copyFrom, &lastFinSet, true );
 }
 
+
 /* Fsm concatentation worker. Supports treating the concatentation as optional,
  * which essentially leaves the final states of machine one as final. */
 void FsmAp::doConcat( FsmAp *other, StateSet *fromStates, bool optional )
@@ -355,14 +362,14 @@ void FsmAp::doConcat( FsmAp *other, StateSet *fromStates, bool optional )
 		StateAp *state = fromStates->data[i];
 
 		/* Merge the former final state with other's start state. */
-		mergeStates( md, state, otherStartState, true );
+		mergeStatesLeaving( md, state, otherStartState );
 
 		/* If the former final state was not reset final then we must clear
 		 * the state's out trans data. If it got reset final then it gets to
 		 * keep its out trans data. This must be done before fillInStates gets
 		 * called to prevent the data from being sourced. */
-		if ( !(state->stateBits & SB_ISFINAL) )
-			relinquishFinal( state );
+		if ( ! state->isFinState() )
+			clearOutData( state );
 	}
 
 	/* Fill in any new states made from merging. */
@@ -557,7 +564,7 @@ void FsmAp::shadowReadWriteStates( MergeData &md )
 					 * there, create it. */
 					if ( targ->isolatedShadow == 0 ) {
 						StateAp *shadow = addState();
-						mergeStates( md, shadow, targ, false );
+						mergeStates( md, shadow, targ );
 						targ->isolatedShadow = shadow;
 					}
 
@@ -585,8 +592,12 @@ void FsmAp::resolveEpsilonTrans( MergeData &md )
 		/* If there is a state vector, then create the pre-merge state. */
 		if ( st->eptVect != 0 ) {
 			/* Merge all the epsilon targets into the state. */
-			for ( EptVect::Iter ept = *st->eptVect; ept.lte(); ept++ )
-				mergeStates( md, st, ept->targ, ept->leaving );
+			for ( EptVect::Iter ept = *st->eptVect; ept.lte(); ept++ ) {
+				if ( ept->leaving )
+					mergeStatesLeaving( md, st, ept->targ );
+				else
+					mergeStates( md, st, ept->targ );
+			}
 
 			/* Clean up the target list. */
 			delete st->eptVect;
@@ -688,7 +699,7 @@ void FsmAp::joinOp( int startId, int finalId, FsmAp **others, int numOthers )
 	}
 
 	/* Take a copy of the final state set, before unsetting them all. This
-	 * will allow us to call relinquishFinal on the states that don't get
+	 * will allow us to call clearOutData on the states that don't get
 	 * final state status back back. */
 	StateSet finStateSetCopy = finStateSet;
 
@@ -715,7 +726,7 @@ void FsmAp::joinOp( int startId, int finalId, FsmAp **others, int numOthers )
 	 * final state status back. */
 	for ( StateSet::Iter st = finStateSetCopy; st.lte(); st++ ) {
 		if ( !((*st)->stateBits & SB_ISFINAL) )
-			relinquishFinal( *st );
+			clearOutData( *st );
 	}
 
 	/* Fill in any new states made from merging. */
@@ -783,7 +794,7 @@ void FsmAp::deterministicEntry()
 			 * all the targets of entry points. */
 			StateAp *newEntry = addState();
 			for ( int en = enId; en < highId; en++ )
-				mergeStates( md, newEntry, prevEntry[en].value, false );
+				mergeStates( md, newEntry, prevEntry[en].value );
 
 			/* Add the new state as the single entry point. */
 			setEntry( prevEntry[enId].key, newEntry );
@@ -866,7 +877,7 @@ void FsmAp::isolateStartState( )
 	setStartState( addState() );
 
 	/* Merge the new start state with the old one to isolate it. */
-	mergeStates( md, startState, prevStartState, false );
+	mergeStates( md, startState, prevStartState );
 
 	/* Stfil and stateDict will be empty because the merging of the old start
 	 * state into the new one will not have any conflicting transitions. */
@@ -879,278 +890,338 @@ void FsmAp::isolateStartState( )
 	setMisfitAccounting( false );
 }
 
-void printCondVals( long key, long values, const CondSet &condSet )
-{
 #ifdef LOG_CONDS
-	cerr << "key " << key << " ";
-	for ( CondSet::Iter csi = condSet; csi.lte(); csi++ ) {
-		bool set = values & (1 << csi.pos());
-		if ( !set )
-			cerr << "!";
-		cerr << (*csi)->name;
-		if ( !csi.last() )
-			cerr << ", ";
+void logCondSpace( CondSpace *condSpace )
+{
+	if ( condSpace == 0 )
+		cerr << "<empty>";
+	else {
+		for ( CondSet::Iter csi = condSpace->condSet.last(); csi.gtb(); csi-- ) {
+			if ( ! csi.last() )
+				cerr << ',';
+			(*csi)->actionName( cerr );
+		}
 	}
+}
+
+void logNewExpansion( Expansion *exp )
+{
+	cerr << "created expansion:" << endl;
+	cerr << "  range: " << exp->lowKey.getVal() << " .. " << 
+			exp->highKey.getVal() << endl;
+
+	cerr << "  fromCondSpace: ";
+	logCondSpace( exp->fromCondSpace );
 	cerr << endl;
+	cerr << "  fromVals: " << exp->fromVals << endl;
+
+	cerr << "  toCondSpace: ";
+	logCondSpace( exp->toCondSpace );
+	cerr << endl;
+	cerr << "  toValsList: ";
+	for ( LongVect::Iter to = exp->toValsList; to.lte(); to++ )
+		cerr << " " << *to;
+	cerr << endl;
+}
 #endif
-}
 
-void FsmAp::makeCondTrans( StateCondDict &mergedCondDict,
-		Expansions &expansions, StateAp *state )
+
+void FsmAp::findTransExpansions( ExpansionList &expansionList, 
+		StateAp *destState, StateAp *srcState )
 {
-	CondSet emptyCondSet;
+	PairIter<TransAp, StateCond> transCond( destState->outList.head,
+			srcState->stateCondList.head );
+	for ( ; !transCond.end(); transCond++ ) {
+		if ( transCond.userState == RangeOverlap ) {
+			Expansion *expansion = new Expansion( transCond.s1Tel.lowKey, 
+					transCond.s1Tel.highKey );
+			expansion->fromTrans = new TransAp(*transCond.s1Tel.trans);
+			expansion->fromTrans->fromState = 0;
+			expansion->fromTrans->toState = transCond.s1Tel.trans->toState;
+			expansion->fromCondSpace = 0;
+			expansion->fromVals = 0;
+			CondSpace *srcCS = transCond.s2Tel.trans->condSpace;
+			expansion->toCondSpace = srcCS;
 
-	StateCondDict::Iter mergedCond = mergedCondDict;
-	TransList::Iter trans = state->outList;
-	while ( true ) {
-		if ( trans.end() || mergedCond.end() )
-			break;
-		else if ( keyOps->lt( trans->highKey, mergedCond->key ) )
-			trans.increment();
-		else if ( keyOps->lt( mergedCond->key, trans->lowKey ) )
-			mergedCond.increment();
-		else {
-			/* mergedCond->key is inside the trans. Process the overlap and skip
-			 * over the mergedCond. */
-			CondSet &mergedCS = mergedCond->condSet;
-			int numNew = 1 << mergedCS.length();
+			long numTargVals = (1 << srcCS->condSet.length());
+			for ( long targVals = 0; targVals < numTargVals; targVals++ )
+				expansion->toValsList.append( targVals );
 
 			#ifdef LOG_CONDS
-			cerr << "found an expansion for base key " << mergedCond->key << endl;
-			cerr << "numNew = " << numNew << endl;
+			logNewExpansion( expansion );
 			#endif
-
-			LongVect *expand = new LongVect( mergedCond->key );
-			expand->appendDup( 0, numNew );
-
-			long bk = baseKey( mergedCond->key, mergedCS );
-			for ( CondSet::Iter cim = mergedCS; cim.lte(); cim++ ) {
-				int bitPos = cim.pos();
-
-				for ( int n = 0; n < numNew; n++ ) {
-					bool isSet = n & (1 << bitPos);
-					if ( isSet )
-						expand->data[n+1] |= (1 << bitPos);
-				}
-			}
-
-			for ( int n = 1; n <= numNew; n++ )
-				printCondVals( bk+expand->data[n], expand->data[n], mergedCS );
-			for ( int n = 1; n <= numNew; n++ )
-				expand->data[n] += bk;
-			expansions.append( expand );
-
-			mergedCond.increment();
+			expansionList.append( expansion );
 		}
 	}
+}
 
-	/* Skip to the first transition that may have a condition char. */
-	trans = state->outList;
-	while ( trans.lte() && keyOps->lteq( trans->highKey, keyOps->highKey ) )
-		trans.increment();
+void FsmAp::findCondExpInTrans( ExpansionList &expansionList, StateAp *state, 
+		Key lowKey, Key highKey, CondSpace *fromCondSpace, CondSpace *toCondSpace,
+		long fromVals, LongVect &toValsList )
+{
+	TransAp searchTrans;
+	searchTrans.lowKey = fromCondSpace->baseKey + fromVals * keyOps->alphSize() + 
+			(lowKey - keyOps->minKey);
+	searchTrans.highKey = fromCondSpace->baseKey + fromVals * keyOps->alphSize() + 
+			(highKey - keyOps->minKey);
+	searchTrans.prev = searchTrans.next = 0;
 
-	/* Compute the new transitions created by condition character clashes. */
-	for ( ; trans.lte(); trans++ ) {
-		Condition *cond = 0;
-		CondSetMapEl *csMapEl = 0;
+	PairIter<TransAp> pairIter( state->outList.head, &searchTrans );
+	for ( ; !pairIter.end(); pairIter++ ) {
+		if ( pairIter.userState == RangeOverlap ) {
+			Expansion *expansion = new Expansion( lowKey, highKey );
+			expansion->fromTrans = new TransAp(*pairIter.s1Tel.trans);
+			expansion->fromTrans->fromState = 0;
+			expansion->fromTrans->toState = pairIter.s1Tel.trans->toState;
+			expansion->fromCondSpace = fromCondSpace;
+			expansion->fromVals = fromVals;
+			expansion->toCondSpace = toCondSpace;
+			expansion->toValsList = toValsList;
 
-		long key = trans->lowKey;
-		while (true) {
-			if ( findCondition( key, cond, csMapEl ) ) {
+			expansionList.append( expansion );
+			#ifdef LOG_CONDS
+			logNewExpansion( expansion );
+			#endif
+		}
+	}
+}
+
+void FsmAp::findCondExpansions( ExpansionList &expansionList, 
+		StateAp *destState, StateAp *srcState )
+{
+	PairIter<StateCond, StateCond> condCond( destState->stateCondList.head,
+			srcState->stateCondList.head );
+	for ( ; !condCond.end(); condCond++ ) {
+		if ( condCond.userState == RangeOverlap ) {
+			/* Loop over all existing condVals . */
+			CondSet &destCS = condCond.s1Tel.trans->condSpace->condSet;
+			long destLen = destCS.length();
+
+			/* Find the items in src cond set that are not in dest
+			 * cond set. These are the items that we must expand. */
+			CondSet srcOnlyCS = condCond.s2Tel.trans->condSpace->condSet;
+			for ( CondSet::Iter dcsi = destCS; dcsi.lte(); dcsi++ )
+				srcOnlyCS.remove( *dcsi );
+			long srcOnlyLen = srcOnlyCS.length();
+
+			if ( srcOnlyCS.length() > 0 ) {
 				#ifdef LOG_CONDS
-				cerr << "found a condition for ";
+				cerr << "there are " << srcOnlyCS.length() << " item(s) that are "
+							"only in the srcCS" << endl;
 				#endif
-				long values = key - csMapEl->baseKey;
-				printCondVals( key, values, csMapEl->condSet );
 
-				/* Get the condSet in the state. */
-				StateCond *srcCond = state->stateCondDict.find( cond->key );
-				CondSet &srcCS = srcCond != 0 ? srcCond->condSet : emptyCondSet;
+				CondSet mergedCS = destCS;
+				mergedCS.insert( condCond.s2Tel.trans->condSpace->condSet );
 
-				/* Get the condSet in the merged state. */
-				StateCond *mergedCond = mergedCondDict.find( cond->key );
-				CondSet &mergedCS = mergedCond != 0 ? mergedCond->condSet : emptyCondSet;
-				CondSet diffCS = mergedCS;
+				CondSpace *fromCondSpace = addCondSpace( destCS );
+				CondSpace *toCondSpace = addCondSpace( mergedCS );
 
-				/* Remove items that are in the srcCS from items that are
-				 * in the mergedCS. */
-				for ( CondSet::Iter ci = srcCS; ci.lte(); ci++ )
-					diffCS.remove( *ci );
-
-				if ( diffCS.length() > 0 ) {
-					#ifdef LOG_CONDS
-					cerr << "there are " << diffCS.length() << " item(s) in the "
-							"mergedCS that are not in the srcCS" << endl;
-					#endif
-					long basic = 0;
-					for ( CondSet::Iter cis = csMapEl->condSet; cis.lte(); cis++ ) {
-						if ( values & (1 << cis.pos()) ) {
-							Action **cim = mergedCS.find( *cis );
-							int bitPos = (cim - mergedCS.data);
-							basic |= 1 << bitPos;
+				/* Loop all values in the dest space. */
+				for ( long destVals = 0; destVals < (1 << destLen); destVals++ ) {
+					long basicVals = 0;
+					for ( CondSet::Iter csi = destCS; csi.lte(); csi++ ) {
+						if ( destVals & (1 << csi.pos()) ) {
+							Action **cim = mergedCS.find( *csi );
+							long bitPos = (cim - mergedCS.data);
+							basicVals |= 1 << bitPos;
 						}
 					}
 
-					int numNew = 1 << diffCS.length();
-					LongVect *expand = new LongVect( key );
-					expand->appendDup( basic, numNew );
-
-					long bk = baseKey( cond->key, mergedCS );
-					for ( CondSet::Iter cid = diffCS; cid.lte(); cid++ ) {
-						Action **cim = mergedCS.find( *cid );
-						int bitPos = (cim - mergedCS.data);
-						int newPos = cid.pos();
-
-						for ( int n = 0; n < numNew; n++ ) {
-							bool isSet = n & (1 << newPos);
-							if ( isSet )
-								expand->data[n+1] |= (1 << bitPos);
+					/* Loop all new values. */
+					LongVect expandToVals;
+					for ( long soVals = 0; soVals < (1 << srcOnlyLen); soVals++ ) {
+						long targVals = basicVals;
+						for ( CondSet::Iter csi = srcOnlyCS; csi.lte(); csi++ ) {
+							if ( soVals & (1 << csi.pos()) ) {
+								Action **cim = mergedCS.find( *csi );
+								long bitPos = (cim - mergedCS.data);
+								targVals |= 1 << bitPos;
+							}
 						}
+						expandToVals.append( targVals );
 					}
 
-					for ( int n = 1; n <= numNew; n++ )
-						printCondVals( bk+expand->data[n], expand->data[n], mergedCS );
-					for ( int n = 1; n <= numNew; n++ )
-						expand->data[n] += bk;
-					expansions.append( expand );
+					findCondExpInTrans( expansionList, destState, 
+							condCond.s1Tel.lowKey, condCond.s1Tel.highKey, 
+							fromCondSpace, toCondSpace, destVals, expandToVals );
 				}
 			}
+		}
+	}
+}
 
-			if ( keyOps->eq( key, trans->highKey ) )
+void FsmAp::doExpand( MergeData &md, StateAp *destState, ExpansionList &expList1 )
+{
+	for ( ExpansionList::Iter exp = expList1; exp.lte(); exp++ ) {
+		for ( LongVect::Iter to = exp->toValsList; to.lte(); to++ ) {
+			long targVals = *to;
+
+			/* We will use the copy of the transition that was made when the
+			 * expansion was created. It will get used multiple times. Each
+			 * time we must set up the keys, everything else is constant and
+			 * and already prepared. */
+			TransAp *srcTrans = exp->fromTrans;
+
+			srcTrans->lowKey = exp->toCondSpace->baseKey +
+					targVals * keyOps->alphSize() + (exp->lowKey - keyOps->minKey);
+			srcTrans->highKey = exp->toCondSpace->baseKey +
+					targVals * keyOps->alphSize() + (exp->highKey - keyOps->minKey);
+
+			TransList srcList;
+			srcList.append( srcTrans );
+			outTransCopy( md, destState, srcList.head );
+			srcList.abandon();
+		}
+	}
+}
+
+
+void FsmAp::doRemove( MergeData &md, StateAp *destState, ExpansionList &expList1 )
+{
+	for ( ExpansionList::Iter exp = expList1; exp.lte(); exp++ ) {
+		Removal removal;
+		if ( exp->fromCondSpace == 0 ) {
+			removal.lowKey = exp->lowKey;
+			removal.highKey = exp->highKey;
+		}
+		else {
+			removal.lowKey = exp->fromCondSpace->baseKey + 
+				exp->fromVals * keyOps->alphSize() + (exp->lowKey - keyOps->minKey);
+			removal.highKey = exp->fromCondSpace->baseKey + 
+				exp->fromVals * keyOps->alphSize() + (exp->highKey - keyOps->minKey);
+		}
+		removal.next = 0;
+
+		TransList destList;
+		PairIter<TransAp, Removal> pairIter( destState->outList.head, &removal );
+		for ( ; !pairIter.end(); pairIter++ ) {
+			switch ( pairIter.userState ) {
+			case RangeInS1: {
+				TransAp *destTrans = pairIter.s1Tel.trans;
+				destTrans->lowKey = pairIter.s1Tel.lowKey;
+				destTrans->highKey = pairIter.s1Tel.highKey;
+				destList.append( destTrans );
+				break;
+			}
+			case RangeInS2:
+				break;
+			case RangeOverlap: {
+				TransAp *trans = pairIter.s1Tel.trans;
+				detachTrans( trans->fromState, trans->toState, trans );
+				delete trans;
+				break;
+			}
+			case BreakS1: {
+				pairIter.s1Tel.trans = dupTrans( destState, 
+						pairIter.s1Tel.trans );
+				break;
+			}
+			case BreakS2:
 				break;
-			keyOps->inc( key );
-		}
-	}
-}
-
-
-void FsmAp::doExpand( StateAp *newState, StateAp *srcState, LongVect *expansion )
-{
-	#ifdef LOG_CONDS
-	cerr << "expansion of " << expansion->data[0] << " to";
-	for ( int to = 1; to < expansion->length(); to++ )
-		cerr << " " << expansion->data[to];
-	cerr << endl;
-	#endif
-
-	for ( int i = 2; i < expansion->length(); i++ )
-		assert( keyOps->lt( expansion->data[i-1], expansion->data[i] ) );
-
-	bool covers = false;
-	long key = expansion->data[0];
-	for ( TransList::Iter trans = srcState->outList; trans.lte(); trans++ ) {
-		if ( keyOps->eq( key, trans->lowKey ) ||
-			keyOps->lt( trans->lowKey, key ) && keyOps->lt( key, trans->highKey ) ||
-			keyOps->eq( key, trans->highKey ) )
-		{
-			for ( int to = 1; to < expansion->length(); to++ ) {
-				/* Dup src's trans into the new transition element. */
-				TransAp *newTrans = dupTrans( newState, trans, false );
-				newTrans->lowKey = newTrans->highKey = expansion->data[to];
-				appendTrans( newState->outList, newTrans, covers );
 			}
-			return;
 		}
+		destState->outList.transfer( destList );
 	}
-	assert( false );
 }
 
-void FsmAp::removeExpSource( StateAp *destState, LongVect *expansion )
+void FsmAp::mergeStateConds( StateAp *destState, StateAp *srcState )
 {
-	assert( misfitAccounting );
-	long key = expansion->data[0];
+	StateCondList destList;
+	PairIter<StateCond> pairIter( destState->stateCondList.head,
+			srcState->stateCondList.head );
+	for ( ; !pairIter.end(); pairIter++ ) {
+		switch ( pairIter.userState ) {
+		case RangeInS1: {
+			StateCond *destCond = pairIter.s1Tel.trans;
+			destCond->lowKey = pairIter.s1Tel.lowKey;
+			destCond->highKey = pairIter.s1Tel.highKey;
+			destList.append( destCond );
+			break;
+		}
+		case RangeInS2: {
+			StateCond *newCond = new StateCond( *pairIter.s2Tel.trans );
+			newCond->lowKey = pairIter.s2Tel.lowKey;
+			newCond->highKey = pairIter.s2Tel.highKey;
+			destList.append( newCond );
+			break;
+		}
+		case RangeOverlap: {
+			StateCond *destCond = pairIter.s1Tel.trans;
+			StateCond *srcCond = pairIter.s2Tel.trans;
+			CondSet mergedCondSet;
+			mergedCondSet.insert( destCond->condSpace->condSet );
+			mergedCondSet.insert( srcCond->condSpace->condSet );
+			destCond->condSpace = addCondSpace( mergedCondSet );
 
-	for ( TransList::Iter trans = destState->outList; trans.lte(); ) {
-		TransList::Iter next = trans->next;
+			destCond->lowKey = pairIter.s1Tel.lowKey;
+			destCond->highKey = pairIter.s1Tel.highKey;
+			destList.append( destCond );
+			break;
+		}
+		case BreakS1:
+			pairIter.s1Tel.trans = new StateCond( *pairIter.s1Tel.trans );
+			break;
 
-		if ( keyOps->eq( trans->lowKey, key ) && keyOps->eq( key, trans->highKey ) ) {
-			#ifdef LOG_CONDS
-			cerr << "removing exp source for key = " << key << endl;
-			#endif
-			detachTrans( trans->fromState, trans->toState, trans );
-			destState->outList.detach( trans );
-			delete trans;
+		case BreakS2:
+			break;
 		}
-		else if ( keyOps->eq( trans->lowKey, key ) && keyOps->lt( key, trans->highKey ) ) {
-			/* Unimplemented. */
-			assert( false );
-		}
-		else if ( keyOps->lt( trans->lowKey, key ) && keyOps->eq( key, trans->highKey ) ) {
-			/* Unimplemented. */
-			assert( false );
-		}
-		else if ( keyOps->lt( trans->lowKey, key ) && keyOps->eq( key, trans->highKey ) ) {
-			/* Unimplemented. */
-			assert( false );
-		}
-		trans = next;
+	}
+	destState->stateCondList.transfer( destList );
+}
+
+/* A state merge which represents the drawing in of leaving transitions.  If
+ * there is any out data then we duplicate the souce state, transfer the out
+ * data, then merge in the state. The new state will be reaped because it will
+ * not be given any in transitions. */
+void FsmAp::mergeStatesLeaving( MergeData &md, StateAp *destState, StateAp *srcState )
+{
+	if ( !hasOutData( destState ) )
+		mergeStates( md, destState, srcState );
+	else {
+		StateAp *ssMutable = addState();
+		mergeStates( md, ssMutable, srcState );
+		transferOutData( ssMutable, destState );
+
+		for ( ActionSet::Iter cond = destState->outCondSet; cond.lte(); cond++ )
+			embedCondition( md, ssMutable, *cond );
+
+		mergeStates( md, destState, ssMutable );
 	}
 }
 
 void FsmAp::mergeStates( MergeData &md, StateAp *destState, 
-		StateAp *srcState, bool leaving )
+		StateAp **srcStates, int numSrc )
 {
-	/* Compute the stateCondDict that will go into the destination state. */
-	StateCondDict mergedCondDict( destState->stateCondDict );
-	for ( StateCondDict::Iter srcCond = srcState->stateCondDict; srcCond.lte(); srcCond++ ) {
-		StateCond *destCond = mergedCondDict.find( srcCond->key );
-		if ( destCond == 0 ) {
-			destCond = new StateCond( srcCond->key );
-			mergedCondDict.insert( destCond );
-		}
+	for ( int s = 0; s < numSrc; s++ )
+		mergeStates( md, destState, srcStates[s] );
+}
 
-		for ( CondSet::Iter ca = srcCond->condSet; ca.lte(); ca++ )
-			destCond->condSet.insert( *ca );
+void FsmAp::mergeStates( MergeData &md, StateAp *destState, StateAp *srcState )
+{
+	ExpansionList expList1;
+	ExpansionList expList2;
+
+	findTransExpansions( expList1, destState, srcState );
+	findCondExpansions( expList1, destState, srcState );
+	findTransExpansions( expList2, srcState, destState );
+	findCondExpansions( expList2, srcState, destState );
+
+	mergeStateConds( destState, srcState );
 	
-		addCondition( destCond->key, destCond->condSet );
-	}
+	outTransCopy( md, destState, srcState->outList.head );
 
-	/* Check for expansions in the source state. If there are expansions, then
-	 * we must make a copy of it. The source state is read-only. */
-	Expansions expns1;
-	makeCondTrans( mergedCondDict, expns1, srcState );
+	doExpand( md, destState, expList1 );
+	doExpand( md, destState, expList2 );
 
-	/* Check for expansions in the destination state. We can make these
-	 * directly. */
-	Expansions expns2;
-	makeCondTrans( mergedCondDict, expns2, destState );
+	doRemove( md, destState, expList1 );
+	doRemove( md, destState, expList2 );
 
-	/* Copy in the new state cond dict. */
-	destState->stateCondDict.empty();
-	destState->stateCondDict.transfer( mergedCondDict );
-
-	/* Get the out transitions. */
-	outTransCopy( md, destState, srcState, leaving );
-
-	if ( expns1.length() > 0 ) {
-		for ( Expansions::Iter exp = expns1; exp.lte(); exp++ ) {
-			StateAp *newState = addState();
-			doExpand( newState, srcState, *exp );
-			outTransCopy( md, destState, newState, false );
-		}
-	}
-
-	if ( expns2.length() > 0 ) {
-		for ( Expansions::Iter exp = expns2; exp.lte(); exp++ ) {
-			StateAp *newState = addState();
-			doExpand( newState, destState, *exp );
-			outTransCopy( md, destState, newState, false );
-		}
-	}
-
-	if ( expns1.length() > 0 ) {
-		for ( Expansions::Iter exp = expns1; exp.lte(); exp++ )
-			removeExpSource( destState, *exp );
-	}
-
-	if ( expns2.length() > 0 ) {
-		for ( Expansions::Iter exp = expns2; exp.lte(); exp++ )
-			removeExpSource( destState, *exp );
-	}
+	expList1.empty();
+	expList2.empty();
 
 	/* Get its bits and final state status. */
 	destState->stateBits |= ( srcState->stateBits & ~SB_ISFINAL );
-	if ( srcState->stateBits & SB_ISFINAL )
+	if ( srcState->isFinState() )
 		setFinState( destState );
 
 	/* Draw in any properties of srcState into destState. */
@@ -1160,12 +1231,13 @@ void FsmAp::mergeStates( MergeData &md, StateAp *destState,
 		 * effect. */
 		destState->epsilonTrans.append( EpsilonTrans( srcState->epsilonTrans ) );
 
-		/* Get all actions, again duplicating to protect agains write to source. */
+		/* Get all actions, duplicating to protect against write to source. */
 		destState->toStateActionTable.setActions( 
 				ActionTable( srcState->toStateActionTable ) );
 		destState->fromStateActionTable.setActions( 
 				ActionTable( srcState->fromStateActionTable ) );
 		destState->outActionTable.setActions( ActionTable( srcState->outActionTable ) );
+		destState->outCondSet.insert( ActionSet( srcState->outCondSet ) );
 		destState->errActionTable.setActions( ErrActionTable( srcState->errActionTable ) );
 		destState->eofActionTable.setActions( ActionTable( srcState->eofActionTable ) );
 	}
@@ -1178,18 +1250,12 @@ void FsmAp::mergeStates( MergeData &md, StateAp *destState,
 		destState->toStateActionTable.setActions( srcState->toStateActionTable );
 		destState->fromStateActionTable.setActions( srcState->fromStateActionTable );
 		destState->outActionTable.setActions( srcState->outActionTable );
+		destState->outCondSet.insert( srcState->outCondSet );
 		destState->errActionTable.setActions( srcState->errActionTable );
 		destState->eofActionTable.setActions( srcState->eofActionTable );
 	}
 }
 
-void FsmAp::mergeStates( MergeData &md, StateAp *destState, 
-		StateAp **srcStates, int numSrc )
-{
-	for ( int s = 0; s < numSrc; s++ )
-		mergeStates( md, destState, srcStates[s], false );
-}
-
 void FsmAp::fillInStates( MergeData &md )
 {
 	/* Merge any states that are awaiting merging. This will likey cause
@@ -1212,6 +1278,122 @@ void FsmAp::fillInStates( MergeData &md )
 		state = state->alg.next;
 	}
 
-	/* StateDict will still have it's ptrs/size set but all of it's element
+	/* StateDict will still have its ptrs/size set but all of it's element
 	 * will be deleted so we don't need to clean it up. */
 }
+
+void FsmAp::findEmbedExpansions( ExpansionList &expansionList, 
+		StateAp *destState, Action *condAction )
+{
+	StateCondList destList;
+	PairIter<TransAp, StateCond> transCond( destState->outList.head,
+			destState->stateCondList.head );
+	for ( ; !transCond.end(); transCond++ ) {
+		switch ( transCond.userState ) {
+			case RangeInS1: {
+				if ( transCond.s1Tel.lowKey <= keyOps->maxKey ) {
+					assert( transCond.s1Tel.highKey <= keyOps->maxKey );
+
+					/* Make a new state cond. */
+					StateCond *newStateCond = new StateCond( transCond.s1Tel.lowKey,
+							transCond.s1Tel.highKey );
+					newStateCond->condSpace = addCondSpace( CondSet( condAction ) );
+					destList.append( newStateCond );
+
+					/* Create the expansion. */
+					Expansion *expansion = new Expansion( transCond.s1Tel.lowKey,
+							transCond.s1Tel.highKey );
+					expansion->fromTrans = new TransAp(*transCond.s1Tel.trans);
+					expansion->fromTrans->fromState = 0;
+					expansion->fromTrans->toState = transCond.s1Tel.trans->toState;
+					expansion->fromCondSpace = 0;
+					expansion->fromVals = 0;
+					expansion->toCondSpace = newStateCond->condSpace;
+					expansion->toValsList.append( 1 );
+					#ifdef LOG_CONDS
+					logNewExpansion( expansion );
+					#endif
+					expansionList.append( expansion );
+				}
+				break;
+			}
+			case RangeInS2: {
+				/* Enhance state cond and find the expansion. */
+				StateCond *stateCond = transCond.s2Tel.trans;
+				stateCond->lowKey = transCond.s2Tel.lowKey;
+				stateCond->highKey = transCond.s2Tel.highKey;
+
+				CondSet &destCS = stateCond->condSpace->condSet;
+				long destLen = destCS.length();
+				CondSpace *fromCondSpace = stateCond->condSpace;
+
+				CondSet mergedCS = destCS;
+				mergedCS.insert( condAction );
+				CondSpace *toCondSpace = addCondSpace( mergedCS );
+				stateCond->condSpace = toCondSpace;
+				destList.append( stateCond );
+
+				/* Loop all values in the dest space. */
+				for ( long destVals = 0; destVals < (1 << destLen); destVals++ ) {
+					long basicVals = 0;
+					for ( CondSet::Iter csi = destCS; csi.lte(); csi++ ) {
+						if ( destVals & (1 << csi.pos()) ) {
+							Action **cim = mergedCS.find( *csi );
+							long bitPos = (cim - mergedCS.data);
+							basicVals |= 1 << bitPos;
+						}
+					}
+
+					long targVals = basicVals;
+					Action **cim = mergedCS.find( condAction );
+					long bitPos = (cim - mergedCS.data);
+					targVals |= 1 << bitPos;
+					
+					LongVect expandToVals( targVals );
+					findCondExpInTrans( expansionList, destState, 
+						transCond.s2Tel.lowKey, transCond.s2Tel.highKey, 
+						fromCondSpace, toCondSpace, destVals, expandToVals );
+				}
+				break;
+			}
+
+
+			case RangeOverlap:
+			case BreakS1:
+			case BreakS2:
+				assert( false );
+				break;
+		}
+	}
+
+	destState->stateCondList.transfer( destList );
+}
+
+void FsmAp::embedCondition( StateAp *state, Action *condAction )
+{
+	MergeData md;
+	ExpansionList expList;
+
+	/* Turn on misfit accounting to possibly catch the old start state. */
+	setMisfitAccounting( true );
+
+	/* Worker. */
+	embedCondition( md, state, condAction );
+
+	/* Fill in any states that were newed up as combinations of others. */
+	fillInStates( md );
+
+	/* Remove the misfits and turn off misfit accounting. */
+	removeMisfits();
+	setMisfitAccounting( false );
+}
+
+void FsmAp::embedCondition( MergeData &md, StateAp *state, Action *condAction )
+{
+	ExpansionList expList;
+
+	findEmbedExpansions( expList, state, condAction );
+	doExpand( md, state, expList );
+	doRemove( md, state, expList );
+	expList.empty();
+}
