@@ -529,7 +529,7 @@ bool FsmAp::outListCovers( StateAp *state )
 	
 	/* The first must start at the lower bound. */
 	TransList::Iter trans = state->outList.first();
-	if ( keyOps->lt( keyOps->lowKey, trans->lowKey ) )
+	if ( keyOps->minKey < trans->lowKey )
 		return false;
 
 	/* Loop starts at second el. */
@@ -539,15 +539,15 @@ bool FsmAp::outListCovers( StateAp *state )
 	for ( ; trans.lte(); trans++ ) {
 		/* Lower end of the trans must be one greater than the
 		 * previous' high end. */
-		long lowKey = trans->lowKey;
-		keyOps->dec( lowKey );
-		if ( keyOps->lt( trans->prev->highKey, lowKey ) )
+		Key lowKey = trans->lowKey;
+		lowKey.decrement();
+		if ( trans->prev->highKey < lowKey )
 			return false;
 	}
 
 	/* Require that the last range extends to the upper bound. */
 	trans = state->outList.last();
-	if ( keyOps->lt( trans->highKey, keyOps->highKey ) )
+	if ( trans->highKey < keyOps->maxKey )
 		return false;
 
 	return true;
@@ -703,3 +703,30 @@ void FsmAp::fusePartitions( MinPartition *parts, int numParts )
 	}
 }
 
+
+/* Merge neighboring transitions go to the same state and have the same
+ * transitions data. */
+void FsmAp::compressTransitions()
+{
+	for ( StateList::Iter st = stateList; st.lte(); st++ ) {
+		if ( st->outList.length() > 1 ) {
+			for ( TransList::Iter trans = st->outList, next = trans.next(); next.lte();  ) {
+				Key nextLow = next->lowKey;
+				nextLow.decrement();
+				if ( trans->highKey == nextLow && trans->toState == next->toState &&
+					CmpActionTable::compare( trans->actionTable, next->actionTable ) == 0 )
+				{
+					trans->highKey = next->highKey;
+					st->outList.detach( next );
+					detachTrans( next->fromState, next->toState, next );
+					delete next;
+					next = trans.next();
+				}
+				else {
+					trans.increment();
+					next.increment();
+				}
+			}
+		}
+	}
+}
