@@ -1238,21 +1238,22 @@ void ParseData::checkInlineList( Action *act, InlineList *inlineList )
 
 void ParseData::checkAction( Action *action )
 {
+	/* Check for actions with calls that are embedded within a longest match
+	 * machine. */
 	if ( !action->isLmAction && action->numRefs() > 0 && action->anyCall ) {
-		/* Check for actions with calls that are embedded within a longest match
-		 * machine. */
 		for ( ActionRefs::Iter ar = action->actionRefs; ar.lte(); ar++ ) {
 			NameInst *check = *ar;
 			while ( check != 0 ) {
 				if ( check->isLongestMatch ) {
-					error(action->loc) << "fcall permitted only in pattern actions "
-						"of a longest match" << endl;
+					error(action->loc) << "within a scanner, fcall is permitted"
+						" only in pattern actions" << endl;
 					break;
 				}
 				check = check->parent;
 			}
 		}
 	}
+
 	checkInlineList( action, action->inlineList );
 }
 
