@@ -632,18 +632,18 @@ Expression::~Expression()
 }
 
 /* Evaluate a single expression node. */
-FsmAp *Expression::walk( ParseData *pd )
+FsmAp *Expression::walk( ParseData *pd, bool lastInSeq )
 {
 	FsmAp *rtnVal = 0;
 	switch ( type ) {
 		case OrType: {
 			/* Evaluate the expression. */
-			rtnVal = expression->walk( pd );
+			rtnVal = expression->walk( pd, false );
 			/* Evaluate the term. */
 			FsmAp *rhs = term->walk( pd );
 			/* Perform union. */
 			rtnVal->unionOp( rhs );
-			afterOpMinimize( rtnVal );
+			afterOpMinimize( rtnVal, lastInSeq );
 			break;
 		}
 		case IntersectType: {
@@ -653,7 +653,7 @@ FsmAp *Expression::walk( ParseData *pd )
 			FsmAp *rhs = term->walk( pd );
 			/* Perform intersection. */
 			rtnVal->intersectOp( rhs );
-			afterOpMinimize( rtnVal );
+			afterOpMinimize( rtnVal, lastInSeq );
 			break;
 		}
 		case SubtractType: {
@@ -663,7 +663,7 @@ FsmAp *Expression::walk( ParseData *pd )
 			FsmAp *rhs = term->walk( pd );
 			/* Perform subtraction. */
 			rtnVal->subtractOp( rhs );
-			afterOpMinimize( rtnVal );
+			afterOpMinimize( rtnVal, lastInSeq );
 			break;
 		}
 		case StrongSubtractType: {
@@ -679,7 +679,7 @@ FsmAp *Expression::walk( ParseData *pd )
 
 			/* Perform subtraction. */
 			rtnVal->subtractOp( rhs );
-			afterOpMinimize( rtnVal );
+			afterOpMinimize( rtnVal, lastInSeq );
 			break;
 		}
 		case TermType: {
@@ -751,18 +751,18 @@ Term::~Term()
 }
 
 /* Evaluate a term node. */
-FsmAp *Term::walk( ParseData *pd )
+FsmAp *Term::walk( ParseData *pd, bool lastInSeq )
 {
 	FsmAp *rtnVal = 0;
 	switch ( type ) {
 		case ConcatType: {
 			/* Evaluate the Term. */
-			rtnVal = term->walk( pd );
+			rtnVal = term->walk( pd, false );
 			/* Evaluate the FactorWithRep. */
 			FsmAp *rhs = factorWithAug->walk( pd );
 			/* Perform concatenation. */
 			rtnVal->concatOp( rhs );
-			afterOpMinimize( rtnVal );
+			afterOpMinimize( rtnVal, lastInSeq );
 			break;
 		}
 		case RightStartType: {
@@ -786,7 +786,7 @@ FsmAp *Term::walk( ParseData *pd )
 
 			/* Perform concatenation. */
 			rtnVal->concatOp( rhs );
-			afterOpMinimize( rtnVal );
+			afterOpMinimize( rtnVal, lastInSeq );
 			break;
 		}
 		case RightFinishType: {
@@ -811,7 +811,7 @@ FsmAp *Term::walk( ParseData *pd )
 
 			/* Perform concatenation. */
 			rtnVal->concatOp( rhs );
-			afterOpMinimize( rtnVal );
+			afterOpMinimize( rtnVal, lastInSeq );
 			break;
 		}
 		case LeftType: {
@@ -838,7 +838,7 @@ FsmAp *Term::walk( ParseData *pd )
 
 			/* Perform concatenation. */
 			rtnVal->concatOp( rhs );
-			afterOpMinimize( rtnVal );
+			afterOpMinimize( rtnVal, lastInSeq );
 			break;
 		}
 		case FactorWithAugType: {
@@ -1060,6 +1060,28 @@ void FactorWithAug::assignPriorities( FsmAp *graph, int *priorOrd )
 	}
 }
 
+void FactorWithAug::assignConditions( FsmAp *graph )
+{
+	for ( int i = 0; i < conditions.length(); i++ )  {
+		switch ( conditions[i].type ) {
+		/* Transition actions. */
+		case at_start:
+			graph->startFsmCondition( conditions[i].action );
+			afterOpMinimize( graph );
+			break;
+		case at_all:
+			graph->allTransCondition( conditions[i].action );
+			break;
+		case at_leave:
+			graph->leaveFsmCondition( conditions[i].action );
+			break;
+		default:
+			break;
+		}
+	}
+}
+
+
 /* Evaluate a factor with augmentation node. */
 FsmAp *FactorWithAug::walk( ParseData *pd )
 {
@@ -1097,6 +1119,8 @@ FsmAp *FactorWithAug::walk( ParseData *pd )
 			actionOrd[i] = pd->curActionOrd++;
 	}
 
+	assignConditions( rtnVal );
+
 	assignActions( pd, rtnVal , actionOrd );
 
 	/* Make the array of priority orderings. Orderings are local to this walk
@@ -1664,10 +1688,6 @@ FsmAp *Factor::walk( ParseData *pd )
 		break;
 	}
 
-	/* Assign Conditions. */
-	for ( int i = 0; i < conditions.length(); i++ )
-		rtnVal->setCondition( conditions[i] );
-
 	return rtnVal;
 }
 
@@ -1721,11 +1741,11 @@ Range::~Range()
 /* Evaluate a range. Gets the lower an upper key and makes an fsm range. */
 FsmAp *Range::walk( ParseData *pd )
 {
-	long lowKey = lowerLit->makeRangeEndPoint( pd );
-	long highKey = upperLit->makeRangeEndPoint( pd );
+	Key lowKey = lowerLit->makeRangeEndPoint( pd );
+	Key highKey = upperLit->makeRangeEndPoint( pd );
 
 	/* Validate the range. */
-	if ( keyOps->gt( lowKey, highKey ) ) {
+	if ( lowKey > highKey ) {
 		/* Recover by setting upper to lower; */
 		error(lowerLit->loc) << "lower end of range is greater then upper end" << endl;
 		highKey = lowKey;
@@ -1738,9 +1758,9 @@ FsmAp *Range::walk( ParseData *pd )
 }
 
 /* Return an fsm key for a literal that is a range end point. */
-long Literal::makeRangeEndPoint( ParseData *pd )
+Key Literal::makeRangeEndPoint( ParseData *pd )
 {
-	long retVal;
+	Key retVal;
 	switch ( type ) {
 	case Number: {
 		/* Make the fsm key from the number. */
@@ -1764,7 +1784,7 @@ FsmAp *Literal::walk( ParseData *pd )
 	switch ( type ) {
 	case Number: {
 		/* Make the fsm key in int format. */
-		long fsmKey = makeFsmKeyNum( data.data, loc, pd );
+		Key fsmKey = makeFsmKeyNum( data.data, loc, pd );
 		/* Make the new machine. */
 		rtnVal = new FsmAp();
 		rtnVal->concatFsm( fsmKey );
@@ -1772,7 +1792,7 @@ FsmAp *Literal::walk( ParseData *pd )
 	}
 	case LitString: {
 		/* Make the array of keys in int format. */
-		long *arr = new long[data.length];
+		Key *arr = new Key[data.length];
 		makeFsmKeyArray( arr, data.data, data.length, pd );
 
 		/* Make the new machine. */
@@ -1852,7 +1872,7 @@ FsmAp *ReItem::walk( ParseData *pd, RegExpr *rootRegex )
 	switch ( type ) {
 		case Data: {
 			/* Move the data into an integer array and make a concat fsm. */
-			long *arr = new long[data.length];
+			Key *arr = new Key[data.length];
 			makeFsmKeyArray( arr, data.data, data.length, pd );
 
 			/* Make the concat fsm. */
@@ -1954,22 +1974,21 @@ FsmAp *ReOrItem::walk( ParseData *pd, RegExpr *rootRegex )
 		 * keys. Duplicates are silently ignored. The alternative would be to
 		 * issue warning or an error but since we can't with [a0-9a] or 'a' |
 		 * 'a' don't bother here. */
-		int length;
-		long *array = makeFsmUniqueKeyArray( length, data.data, data.length, 
+		KeySet keySet;
+		makeFsmUniqueKeyArray( keySet, data.data, data.length, 
 			rootRegex != 0 ? rootRegex->caseInsensitive : false, pd );
 
 		/* Run the or operator. */
-		rtnVal->orFsm( array, length );
-		delete[] array;
+		rtnVal->orFsm( keySet.data, keySet.length() );
 		break;
 	}
 	case Range: {
 		/* Make the upper and lower keys. */
-		long lowKey = makeFsmKeyChar( lower, pd );
-		long highKey = makeFsmKeyChar( upper, pd );
+		Key lowKey = makeFsmKeyChar( lower, pd );
+		Key highKey = makeFsmKeyChar( upper, pd );
 
 		/* Validate the range. */
-		if ( keyOps->gt( lowKey, highKey ) ) {
+		if ( lowKey > highKey ) {
 			/* Recover by setting upper to lower; */
 			error(loc) << "lower end of range is greater then upper end" << endl;
 			highKey = lowKey;
@@ -1980,9 +1999,9 @@ FsmAp *ReOrItem::walk( ParseData *pd, RegExpr *rootRegex )
 		rtnVal->rangeFsm( lowKey, highKey );
 
 		if ( rootRegex != 0 && rootRegex->caseInsensitive ) {
-			if ( keyOps->lteq( lowKey, 'Z' ) && keyOps->lteq( 'A', highKey ) ) {
-				long otherLow = keyOps->lt( lowKey, 'A' ) ? 'A' : lowKey;
-				long otherHigh = keyOps->lt( 'Z', highKey ) ? 'Z' : highKey;
+			if ( lowKey <= 'Z' && 'A' <= highKey ) {
+				Key otherLow = lowKey < 'A' ? Key('A') : lowKey;
+				Key otherHigh = 'Z' < highKey ? Key('Z') : highKey;
 
 				otherLow = 'a' + ( otherLow - 'A' );
 				otherHigh = 'a' + ( otherHigh - 'A' );
@@ -1992,9 +2011,9 @@ FsmAp *ReOrItem::walk( ParseData *pd, RegExpr *rootRegex )
 				rtnVal->unionOp( otherRange );
 				rtnVal->minimizePartition2();
 			}
-			else if ( keyOps->lteq( lowKey, 'z' ) && keyOps->lteq( 'a', highKey ) ) {
-				long otherLow = keyOps->lt( lowKey, 'a' ) ? 'a' : lowKey;
-				long otherHigh = keyOps->lt( 'z', highKey ) ? 'z' : highKey;
+			else if ( lowKey <= 'z' && 'a' <= highKey ) {
+				Key otherLow = lowKey < 'a' ? Key('a') : lowKey;
+				Key otherHigh = 'z' < highKey ? Key('z') : highKey;
 
 				otherLow = 'A' + ( otherLow - 'a' );
 				otherHigh = 'A' + ( otherHigh - 'a' );
