@@ -32,13 +32,20 @@
 
 /* To what degree are machine minimized. */
 enum MinimizeLevel {
-	MinimizeNone,
 	MinimizeApprox,
 	MinimizeStable,
 	MinimizePartition1,
 	MinimizePartition2
 };
 
+enum MinimizeOpt {
+	MinimizeNone,
+	MinimizeEnd,
+	MinimizeMostOps,
+	MinimizeEveryOp
+};
+
+
 extern FILE *yyin;
 
 /* IO filenames and stream. */
@@ -46,9 +53,10 @@ extern char *outputFileName;
 extern std::ostream *outStream;
 
 /* Options. */
-extern bool minimizeEveryOp;
 extern MinimizeLevel minimizeLevel;
+extern MinimizeOpt minimizeOpt;
 extern char *machineSpec, *machineName;
+extern bool printStatistics;
 
 void yyerror( char *error );
 int yyparse();
@@ -57,10 +65,10 @@ extern int gblErrorCount;
 extern char machineMain[];
 
 /* Error reporting. */
-struct BISON_YYLTYPE;
+struct YYLTYPE;
 struct InputLoc;
 std::ostream &error();
-std::ostream &error( const BISON_YYLTYPE &loc ); 
+std::ostream &error( const YYLTYPE &loc ); 
 std::ostream &error( const InputLoc &loc ); 
 std::ostream &error( int first_line, int first_column );
 std::ostream &warning( ); 
