@@ -90,7 +90,8 @@ private:
 
 	void writeKey( Key key );
 	void writeText( InlineItem *item );
-	void writeWithContext( InlineItem *item, InlineItem *context );
+	void writeCtrlFlow( InlineItem *item, InlineItem *context );
+	void writePtrMod( InlineItem *item, InlineItem *context );
 	void writeGoto( InlineItem *item, InlineItem *context );
 	void writeGotoExpr( InlineItem *item, InlineItem *context );
 	void writeCall( InlineItem *item, InlineItem *context );
@@ -120,6 +121,7 @@ private:
 	void writeLmSwitch( InlineItem *item );
 	void writeMachine();
 	void writeActionExec( InlineItem *item );
+	void writeActionExecTE( InlineItem *item );
 
 	char *fsmName;
 	ParseData *pd;
