@@ -426,14 +426,15 @@ void FlatCodeGen::LOCATE_TRANS()
 
 void FlatCodeGen::GOTO( ostream &ret, int gotoDest, bool inFinish )
 {
-	ret << "{" << CS() << " = " << gotoDest << "; goto _again;}";
+	ret << "{" << CS() << " = " << gotoDest << "; " << 
+			CTRL_FLOW() << "goto _again;}";
 }
 
 void FlatCodeGen::GOTO_EXPR( ostream &ret, InlineItem *ilItem, bool inFinish )
 {
 	ret << "{" << CS() << " = (";
 	INLINE_LIST( ret, ilItem->children, 0, inFinish );
-	ret << "); goto _again;}";
+	ret << "); " << CTRL_FLOW() << "goto _again;}";
 }
 
 void FlatCodeGen::CURS( ostream &ret, bool inFinish )
@@ -461,7 +462,7 @@ void FlatCodeGen::NEXT_EXPR( ostream &ret, InlineItem *ilItem, bool inFinish )
 void FlatCodeGen::CALL( ostream &ret, int callDest, int targState, bool inFinish )
 {
 	ret << "{" << STACK() << "[" << TOP() << "++] = " << CS() << "; " << CS() << " = " << 
-			callDest << "; goto _again;}";
+			callDest << "; " << CTRL_FLOW() << "goto _again;}";
 }
 
 
@@ -469,19 +470,20 @@ void FlatCodeGen::CALL_EXPR( ostream &ret, InlineItem *ilItem, int targState, bo
 {
 	ret << "{" << STACK() << "[" << TOP() << "++] = " << CS() << "; " << CS() << " = (";
 	INLINE_LIST( ret, ilItem->children, targState, inFinish );
-	ret << "); goto _again;}";
+	ret << "); " << CTRL_FLOW() << "goto _again;}";
 }
 
 
 void FlatCodeGen::RET( ostream &ret, bool inFinish )
 {
-	ret << "{" << CS() << " = " << STACK() << "[--" << TOP() << "]; goto _again;}";
+	ret << "{" << CS() << " = " << STACK() << "[--" << TOP() << "]; " << 
+			CTRL_FLOW() << "goto _again;}";
 }
 
 void FlatCodeGen::BREAK( ostream &ret, int targState )
 {
 	outLabelUsed = true;
-	ret << "goto _out;";
+	ret << CTRL_FLOW() << "goto _out;";
 }
 
 void FlatCodeGen::writeOutData()
@@ -607,8 +609,9 @@ void FlatCodeGen::COND_TRANSLATE()
 	for ( CondSpaceList::Iter csi = cgd->condSpaceList; csi.lte(); csi++ ) {
 		CondSpace *condSpace = csi;
 		out << "	case " << condSpace->condSpaceId + 1 << ": {\n";
-		out << TABS(2) << "_widec = " << KEY(condSpace->baseKey) << 
-				" + (" << GET_KEY() << " - " << KEY(keyOps->minKey) << ");\n";
+		out << TABS(2) << "_widec = " << CAST(WIDE_ALPH_TYPE()) << "(" <<
+				KEY(condSpace->baseKey) << " + (" << GET_KEY() << 
+				" - " << KEY(keyOps->minKey) << "));\n";
 
 		for ( CondSet::Iter csi = condSpace->condSet; csi.lte(); csi++ ) {
 			out << TABS(2) << "if ( ";
@@ -621,6 +624,8 @@ void FlatCodeGen::COND_TRANSLATE()
 		out << "		break;\n";
 	}
 
+	SWITCH_DEFAULT();
+
 	out <<
 		"	}\n";
 }
