@@ -1,5 +1,5 @@
 /*
- *  Copyright 2004, 2005 Adrian Thurston <thurston@cs.queensu.ca>
+ *  Copyright 2004-2006 Adrian Thurston <thurston@cs.queensu.ca>
  *            2004 Eric Ocean <eric.ocean@ampede.com>
  *            2005 Alan West <alan@alanz.com>
  */
@@ -57,6 +57,12 @@ protected:
 	std::ostream &TRANS_ACTIONS();
 	void LOCATE_TRANS();
 
+	std::ostream &COND_INDEX_OFFSET();
+	void COND_TRANSLATE();
+	std::ostream &CONDS();
+	std::ostream &COND_KEYS();
+	std::ostream &COND_KEY_SPANS();
+
 	void GOTO( ostream &ret, int gotoDest, bool inFinish );
 	void CALL( ostream &ret, int callDest, int targState, bool inFinish );
 	void NEXT( ostream &ret, int nextDest, bool inFinish );
