@@ -514,6 +514,19 @@ void FsmCodeGen::EXEC( ostream &ret, InlineItem *item, int targState, int inFini
 	ret << "))-1;}";
 }
 
+void FsmCodeGen::EXECTE( ostream &ret, InlineItem *item, int targState, int inFinish )
+{
+	/* Tokend version of exec. */
+
+	/* The parser gives fexec two children. The double brackets are for D
+	 * code. If the inline list is a single word it will get interpreted as a
+	 * C-style cast by the D compiler. */
+	ret << "{" << TOKEND() << " = ((";
+	INLINE_LIST( ret, item->children, targState, inFinish );
+	ret << "));}";
+}
+
+
 void FsmCodeGen::LM_SWITCH( ostream &ret, InlineItem *item, 
 		int targState, int inFinish )
 {
@@ -626,6 +639,12 @@ void FsmCodeGen::INLINE_LIST( ostream &ret, InlineList *inlineList,
 		case InlineItem::Exec:
 			EXEC( ret, item, targState, inFinish );
 			break;
+		case InlineItem::HoldTE:
+			ret << TOKEND() << "--;";
+			break;
+		case InlineItem::ExecTE:
+			EXECTE( ret, item, targState, inFinish );
+			break;
 		case InlineItem::Curs:
 			CURS( ret, inFinish );
 			break;
@@ -838,6 +857,11 @@ std::ostream &CCodeGen::SWITCH_DEFAULT()
 	return out;
 }
 
+string CCodeGen::CTRL_FLOW()
+{
+	return "";
+}
+
 /*
  * D Specific
  */
@@ -896,6 +920,12 @@ std::ostream &DCodeGen::SWITCH_DEFAULT()
 	return out;
 }
 
+string DCodeGen::CTRL_FLOW()
+{
+	return "if (true) ";
+}
+
+
 /* 
  * Java Specific
  */
@@ -943,9 +973,8 @@ string JavaCodeGen::CAST( string type )
 
 string JavaCodeGen::NULL_ITEM()
 {
-	/* Not used. */
-	assert( false );
-	return "0";
+	/* In java we use integers instead of pointers. */
+	return "-1";
 }
 
 string JavaCodeGen::POINTER()
@@ -975,3 +1004,9 @@ string JavaCodeGen::GET_KEY()
 	}
 	return ret.str();
 }
+
+string JavaCodeGen::CTRL_FLOW()
+{
+	return "if (true) ";
+}
+
