@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2005 Adrian Thurston <thurston@cs.queensu.ca>
+ *  Copyright 2001-2006 Adrian Thurston <thurston@cs.queensu.ca>
  *            2004 Eric Ocean <eric.ocean@ampede.com>
  *            2005 Alan West <alan@alanz.com>
  */
@@ -180,6 +180,28 @@ std::ostream &FTabCodeGen::ACTION_SWITCH()
 
 void FTabCodeGen::writeOutData()
 {
+	if ( anyConditions() ) {
+		OPEN_ARRAY( ARRAY_TYPE(maxCondOffset), CO() );
+		COND_OFFSETS();
+		CLOSE_ARRAY() <<
+		"\n";
+
+		OPEN_ARRAY( ARRAY_TYPE(maxCondLen), CL() );
+		COND_LENS();
+		CLOSE_ARRAY() <<
+		"\n";
+
+		OPEN_ARRAY( WIDE_ALPH_TYPE(), CK() );
+		COND_KEYS();
+		CLOSE_ARRAY() <<
+		"\n";
+
+		OPEN_ARRAY( ARRAY_TYPE(maxCondSpaceId), C() );
+		COND_SPACES();
+		CLOSE_ARRAY() <<
+		"\n";
+	}
+
 	OPEN_ARRAY( ARRAY_TYPE(maxKeyOffset), KO() );
 	KEY_OFFSETS();
 	CLOSE_ARRAY() <<
@@ -259,34 +281,36 @@ void FTabCodeGen::writeOutData()
 	}
 
 	out <<
-		"static int " << START() << " = " << START_STATE_ID() << ";\n"
+		"static const int " << START() << " = " << START_STATE_ID() << ";\n"
 		"\n";
 
 	if ( cgd->writeFirstFinal ) {
 		out <<
-			"static int " << FIRST_FINAL() << " = " << FIRST_FINAL_STATE() << ";\n"
+			"static const int " << FIRST_FINAL() << " = " << FIRST_FINAL_STATE() << ";\n"
 			"\n";
 	}
 
 	if ( cgd->writeErr ) {
 		out <<
-			"static int " << ERROR() << " = " << ERROR_STATE() << ";\n"
+			"static const int " << ERROR() << " = " << ERROR_STATE() << ";\n"
 			"\n";
 	}
 }
 
 void FTabCodeGen::writeOutExec()
 {
+	outLabelUsed = false;
+
 	out << 
 		"	{\n"
-		"	int _slen, _rlen";
+		"	int _klen";
 
 	if ( anyRegCurStateRef() )
 		out << ", _ps";
 
 	out <<
 		";\n"
-		"	" << WIDE_ALPH_TYPE() << " *_keys;\n"
+		"	" << PTR_CONST() << WIDE_ALPH_TYPE() << POINTER() << "_keys;\n"
 		"	int _trans;\n";
 
 	if ( anyConditions() )
@@ -294,34 +318,21 @@ void FTabCodeGen::writeOutExec()
 
 	out << "\n";
 
-	out << 
-		"	" << P() << " -= 1;\n"
-		"	goto _resume;\n"
-		"\n";
-
-	/* Head of the loop. */
-	out << "_again:\n";
-
-	if ( anyToStateActions() ) {
+	if ( cgd->hasEnd ) {
+		outLabelUsed = true;
 		out <<
-			"	switch ( " << TSA() << "[" << CS() << "] ) {\n";
-			TO_STATE_ACTION_SWITCH();
-			SWITCH_DEFAULT() <<
-			"	}\n"
-			"\n";
+			"	if ( " << P() << " == " << PE() << " )\n"
+			"		goto _out;\n";
 	}
 
 	out << "_resume:\n";
 
-	if ( cgd->hasEnd ) {
+	if ( redFsm->errState != 0 ) {
+		outLabelUsed = true;
 		out << 
-			"	if ( ++" << P() << " == " << PE() << " )\n"
+			"	if ( " << CS() << " == " << redFsm->errState->id << " )\n"
 			"		goto _out;\n";
 	}
-	else {
-		out << 
-			"	" << P() << " += 1;\n";
-	}
 
 	if ( anyFromStateActions() ) {
 		out <<
@@ -337,12 +348,6 @@ void FTabCodeGen::writeOutExec()
 
 	LOCATE_TRANS();
 
-	if ( redFsm->errState != 0 ) {
-		out << 
-			"	if ( " << CS() << " == " << redFsm->errState->id << " )\n"
-			"		goto _out;\n";
-	}
-
 	out << "_match:\n";
 
 	if ( anyRegCurStateRef() )
@@ -367,9 +372,31 @@ void FTabCodeGen::writeOutExec()
 			"\n";
 	}
 
-	out << "	goto _again;\n";
+	if ( anyRegActions() || anyActionGotos() || anyActionCalls() || anyActionRets() )
+		out << "_again:\n";
 
-	if ( cgd->hasEnd || anyRegBreak() || redFsm->errState != 0 )
+	if ( anyToStateActions() ) {
+		out <<
+			"	switch ( " << TSA() << "[" << CS() << "] ) {\n";
+			TO_STATE_ACTION_SWITCH();
+			SWITCH_DEFAULT() <<
+			"	}\n"
+			"\n";
+	}
+
+	if ( cgd->hasEnd ) {
+		out << 
+			"	if ( ++" << P() << " != " << PE() << " )\n"
+			"		goto _resume;\n";
+	}
+	else {
+		out << 
+			"	" << P() << " += 1;\n"
+			"	goto _resume;\n";
+	}
+
+
+	if ( outLabelUsed )
 		out << "	_out: {}\n";
 
 	out << "	}\n";
