@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005 Adrian Thurston <thurston@cs.queensu.ca>
+ *  Copyright 2005-2006 Adrian Thurston <thurston@cs.queensu.ca>
  */
 
 /*  This file is part of Ragel.
@@ -50,15 +50,13 @@ struct CodeGenData
 		allActions(0),
 		allActionTables(0),
 		allConditions(0),
+		allCondSpaces(0),
 		allStates(0),
 		nameIndex(0),
 		startState(0),
-		alphType(AT_Char),
 		getKeyExpr(0),
 		accessExpr(0),
 		curStateExpr(0),
-		lowKey(0),
-		highKey(0),
 		codeGen(0),
 		wantComplete(wantComplete),
 		writeOps(0),
@@ -84,18 +82,18 @@ struct CodeGenData
 	Action *allActions;
 	RedAction *allActionTables;
 	Condition *allConditions;
+	CondSpace *allCondSpaces;
 	RedStateAp *allStates;
 	NameInst **nameIndex;
 	int startState;
 	ActionList actionList;
 	ConditionList conditionList;
-	AlphType alphType;
+	CondSpaceList condSpaceList;
 	InlineList *getKeyExpr;
 	InlineList *accessExpr;
 	InlineList *curStateExpr;
-	long lowKey, highKey;
 	FsmCodeGen *codeGen;
-	KeyOps keyOps;
+	KeyOps thisKeyOps;
 	bool wantComplete;
 	int writeOps;
 	bool writeData;
@@ -121,7 +119,7 @@ struct CodeGenData
 	void addEntryPoint( char *name, unsigned long entryState );
 	void setFinal( int snum );
 	void initTransList( int snum, unsigned long length );
-	void newTrans( int snum, int tnum, long lowKey, long highKey, 
+	void newTrans( int snum, int tnum, Key lowKey, Key highKey, 
 			long targ, long act );
 	void finishTransList( int snum );
 	void setStateActions( int snum, long toStateAction, 
@@ -130,21 +128,22 @@ struct CodeGenData
 	void setForcedErrorState()
 		{ redFsm->forcedErrorState = true; }
 	
-	void addStateCond( int snum, ulong cond );
-	void initCondList( ulong length );
-	void conditionItem( int cnum, long actionId );
-	void newCondition( int cnum, long key, long baseKey );
+	void initCondSpaceList( ulong length );
+	void condSpaceItem( int cnum, long condActionId );
+	void newCondSpace( int cnum, int condSpaceId, Key baseKey );
 
-	Condition *findCondition( long key );
+	void initStateCondList( int snum, ulong length );
+	void addStateCond( int snum, Key lowKey, Key highKey, long condNum );
+
+	CondSpace *findCondSpace( Key lowKey, Key highKey );
+	Condition *findCondition( Key key );
 
-	bool isAlphSigned();
 	bool setAlphType( char *data );
-	void setKeyOps();
 
 	void makeCodeGen();
 	void generateGraphviz();
 	void resolveTargetStates( InlineList *inlineList );
-	long findMaxKey();
+	Key findMaxKey();
 
 	void generate();
 	void generateCode();
