@@ -240,8 +240,9 @@ void GotoCodeGen::STATE_GOTO_ERROR()
 void GotoCodeGen::COND_TRANSLATE( StateCond *stateCond, int level )
 {
 	CondSpace *condSpace = stateCond->condSpace;
-	out << TABS(level) << "_widec = " << KEY(condSpace->baseKey) << 
-			" + (" << GET_KEY() << " - " << KEY(keyOps->minKey) << ");\n";
+	out << TABS(level) << "_widec = " << CAST(WIDE_ALPH_TYPE()) << "(" <<
+			KEY(condSpace->baseKey) << " + (" << GET_KEY() << 
+			" - " << KEY(keyOps->minKey) << "));\n";
 
 	for ( CondSet::Iter csi = condSpace->condSet; csi.lte(); csi++ ) {
 		out << TABS(level) << "if ( ";
@@ -542,14 +543,15 @@ std::ostream &GotoCodeGen::FINISH_CASES()
 
 void GotoCodeGen::GOTO( ostream &ret, int gotoDest, bool inFinish )
 {
-	ret << "{" << CS() << " = " << gotoDest << "; goto _again;}";
+	ret << "{" << CS() << " = " << gotoDest << "; " << 
+			CTRL_FLOW() << "goto _again;}";
 }
 
 void GotoCodeGen::GOTO_EXPR( ostream &ret, InlineItem *ilItem, bool inFinish )
 {
 	ret << "{" << CS() << " = (";
 	INLINE_LIST( ret, ilItem->children, 0, inFinish );
-	ret << "); goto _again;}";
+	ret << "); " << CTRL_FLOW() << "goto _again;}";
 }
 
 void GotoCodeGen::CURS( ostream &ret, bool inFinish )
@@ -577,25 +579,26 @@ void GotoCodeGen::NEXT_EXPR( ostream &ret, InlineItem *ilItem, bool inFinish )
 void GotoCodeGen::CALL( ostream &ret, int callDest, int targState, bool inFinish )
 {
 	ret << "{" << STACK() << "[" << TOP() << "++] = " << CS() << "; " << CS() << " = " << 
-			callDest << "; goto _again;}";
+			callDest << "; " << CTRL_FLOW() << "goto _again;}";
 }
 
 void GotoCodeGen::CALL_EXPR( ostream &ret, InlineItem *ilItem, int targState, bool inFinish )
 {
 	ret << "{" << STACK() << "[" << TOP() << "++] = " << CS() << "; " << CS() << " = (";
 	INLINE_LIST( ret, ilItem->children, targState, inFinish );
-	ret << "); goto _again;}";
+	ret << "); " << CTRL_FLOW() << "goto _again;}";
 }
 
 void GotoCodeGen::RET( ostream &ret, bool inFinish )
 {
-	ret << "{" << CS() << " = " << STACK() << "[--" << TOP() << "]; goto _again;}";
+	ret << "{" << CS() << " = " << STACK() << "[--" << TOP() << "]; " << 
+			CTRL_FLOW() << "goto _again;}";
 }
 
 void GotoCodeGen::BREAK( ostream &ret, int targState )
 {
 	outLabelUsed = true;
-	ret << "goto _out;";
+	ret << CTRL_FLOW() << "goto _out;";
 }
 
 void GotoCodeGen::writeOutData()
