@@ -248,7 +248,7 @@ void openOutput( char *inputFile )
 		if ( ext != 0 && strcmp( ext, ".rh" ) == 0 )
 			outputFile = fileNameFromStem( inputFile, ".h" );
 		else {
-			char *defExtension;
+			char *defExtension = 0;
 			switch ( hostLangType ) {
 				case CCode: defExtension = ".c"; break;
 				case DCode: defExtension = ".d"; break;
