@@ -616,14 +616,15 @@ void TabCodeGen::LOCATE_TRANS()
 
 void TabCodeGen::GOTO( ostream &ret, int gotoDest, bool inFinish )
 {
-	ret << "{" << CS() << " = " << gotoDest << "; goto _again;}";
+	ret << "{" << CS() << " = " << gotoDest << "; " << 
+			CTRL_FLOW() << "goto _again;}";
 }
 
 void TabCodeGen::GOTO_EXPR( ostream &ret, InlineItem *ilItem, bool inFinish )
 {
 	ret << "{" << CS() << " = (";
 	INLINE_LIST( ret, ilItem->children, 0, inFinish );
-	ret << "); goto _again;}";
+	ret << "); " << CTRL_FLOW() << "goto _again;}";
 }
 
 void TabCodeGen::CURS( ostream &ret, bool inFinish )
@@ -651,25 +652,26 @@ void TabCodeGen::NEXT_EXPR( ostream &ret, InlineItem *ilItem, bool inFinish )
 void TabCodeGen::CALL( ostream &ret, int callDest, int targState, bool inFinish )
 {
 	ret << "{" << STACK() << "[" << TOP() << "++] = " << CS() << "; " << CS() << " = " << 
-			callDest << "; goto _again;}";
+			callDest << "; " << CTRL_FLOW() << "goto _again;}";
 }
 
 void TabCodeGen::CALL_EXPR( ostream &ret, InlineItem *ilItem, int targState, bool inFinish )
 {
 	ret << "{" << STACK() << "[" << TOP() << "++] = " << CS() << "; " << CS() << " = (";
 	INLINE_LIST( ret, ilItem->children, targState, inFinish );
-	ret << "); goto _again;}";
+	ret << "); " << CTRL_FLOW() << "goto _again;}";
 }
 
 void TabCodeGen::RET( ostream &ret, bool inFinish )
 {
-	ret << "{" << CS() << " = " << STACK() << "[--" << TOP() << "]; goto _again;}";
+	ret << "{" << CS() << " = " << STACK() << "[--" << 
+			TOP() << "]; " << CTRL_FLOW() <<  "goto _again;}";
 }
 
 void TabCodeGen::BREAK( ostream &ret, int targState )
 {
 	outLabelUsed = true;
-	ret << "goto _out;";
+	ret << CTRL_FLOW() << "goto _out;";
 }
 
 void TabCodeGen::writeOutData()
@@ -823,8 +825,9 @@ void TabCodeGen::COND_TRANSLATE()
 	for ( CondSpaceList::Iter csi = cgd->condSpaceList; csi.lte(); csi++ ) {
 		CondSpace *condSpace = csi;
 		out << "	case " << condSpace->condSpaceId << ": {\n";
-		out << TABS(2) << "_widec = " << KEY(condSpace->baseKey) << 
-				" + (" << GET_KEY() << " - " << KEY(keyOps->minKey) << ");\n";
+		out << TABS(2) << "_widec = " << CAST(WIDE_ALPH_TYPE()) << "(" <<
+				KEY(condSpace->baseKey) << " + (" << GET_KEY() << 
+				" - " << KEY(keyOps->minKey) << "));\n";
 
 		for ( CondSet::Iter csi = condSpace->condSet; csi.lte(); csi++ ) {
 			out << TABS(2) << "if ( ";
@@ -837,6 +840,9 @@ void TabCodeGen::COND_TRANSLATE()
 			"		break;\n"
 			"	}\n";
 	}
+
+	SWITCH_DEFAULT();
+
 	out << 
 		"				}\n"
 		"				break;\n"
