@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001, 2002, 2005 Adrian Thurston <thurston@cs.queensu.ca>
+ *  Copyright 2001-2006 Adrian Thurston <thurston@cs.queensu.ca>
  *            2004 Eric Ocean <eric.ocean@ampede.com>
  *            2005 Alan West <alan@alanz.com>
  */
@@ -49,10 +49,14 @@ protected:
 	std::ostream &EOF_ACTION_SWITCH();
 	std::ostream &ACTION_SWITCH();
 
+	std::ostream &COND_KEYS();
+	std::ostream &COND_SPACES();
 	std::ostream &KEYS();
 	std::ostream &INDICIES();
+	std::ostream &COND_OFFSETS();
 	std::ostream &KEY_OFFSETS();
 	std::ostream &INDEX_OFFSETS();
+	std::ostream &COND_LENS();
 	std::ostream &SINGLE_LENS();
 	std::ostream &RANGE_LENS();
 	std::ostream &TO_STATE_ACTIONS();
@@ -64,6 +68,8 @@ protected:
 	std::ostream &TRANS_ACTIONS_WI();
 	void LOCATE_TRANS();
 
+	void COND_TRANSLATE();
+
 	void GOTO( ostream &ret, int gotoDest, bool inFinish );
 	void CALL( ostream &ret, int callDest, int targState, bool inFinish );
 	void NEXT( ostream &ret, int nextDest, bool inFinish );
